<?php

require_once DIR_CATALOG.'model/extension/payment/yoomoney/autoload.php';

use YooKassa\Client;
use YooKassa\Model\PaymentMethodType;
use YooMoneyModule\Model\ApiClient;
use YooMoneyModule\Model\KassaModel;

class YooMoneyKassaModel extends KassaModel
{
    private $invoiceEnable;
    private $invoiceSubject;
    private $invoiceMessage;
    private $invoiceLogo;
    /** @var ApiClient */
    private $apiClient;

    public function __construct(Config $config)
    {
        parent::__construct($config);

        $this->invoiceEnable  = (bool)$config->get('yoomoney_kassa_invoice');
        $this->invoiceSubject = $config->get('yoomoney_kassa_invoice_subject');
        $this->invoiceMessage = $config->get('yoomoney_kassa_invoice_message');
        $this->invoiceLogo    = (bool)$config->get('yoomoney_kassa_invoice_logo');
    }

    /**
     * @param $apiClient
     */
    public function setApiClient($apiClient)
    {
        $this->apiClient = $apiClient;
        $this->apiClient->setKassaModel($this);
    }

    public function setIsEnabled($value)
    {
        $this->enabled = $value ? true : false;
    }

    public function setShopId($value)
    {
        $this->shopId = $value;
    }

    public function setPassword($value)
    {
        $this->password = $value;
    }

    public function setEpl($value)
    {
        $this->epl = $value ? true : false;
    }

    public function setUseInstallmentsButton($value)
    {
        $this->useInstallmentsButton = (bool)$value;
    }

    public function setPaymentMethodFlag($paymentMethod, $value)
    {
        if (array_key_exists($paymentMethod, $this->paymentMethods)) {
            $this->paymentMethods[$paymentMethod] = $value ? true : false;
        }
    }

    public function setSendReceipt($value)
    {
        $this->isSendReceipt = $value ? true : false;
    }

    public function setDefaultTaxRate($value)
    {
        if (!in_array($value, $this->getTaxRateList())) {
            $value = 1;
        }
        $this->defaultTaxRate = (int)$value;
    }

    public function setDefaultTaxSystemCode($value)
    {
        if (!in_array($value, $this->getTaxSystemCodeList())) {
            $value = 0;
        }
        $this->defaultTaxSystemCode = (int)$value;
    }

    public function setTaxRates($taxRates)
    {
        $all            = $this->getTaxRateList();
        $this->taxRates = array();
        foreach ($taxRates as $shopTaxRateId => $taxRate) {
            if (in_array($taxRate, $all)) {
                $this->taxRates[$shopTaxRateId] = (int)$taxRate;
            }
        }
    }

    public function setSuccessOrderStatusId($value)
    {
        $this->successOrderStatus = (int)$value;
    }

    public function setMinPaymentAmount($value)
    {
        if ($value < 0) {
            $value = 0;
        }
        $this->minPaymentAmount = (int)$value;
    }

    public function setGeoZoneId($value)
    {
        $this->geoZone = $value;
    }

    public function setDebugLog($value)
    {
        $this->log = $value ? true : false;
    }

    public function setDisplayName($value)
    {
        $this->displayName = $value;
    }

    /**
     * @return bool
     */
    public function isInvoicesEnabled()
    {
        return $this->invoiceEnable;
    }

    /**
     * @param bool $value
     */
    public function setInvoicesEnabled($value)
    {
        $this->invoiceEnable = $value;
    }

    /**
     * @return string
     */
    public function getInvoiceSubject()
    {
        return $this->invoiceSubject;
    }

    /**
     * @param string $value
     */
    public function setInvoiceSubject($value)
    {
        $this->invoiceSubject = $value;
    }

    /**
     * @return string
     */
    public function getInvoiceMessage()
    {
        return $this->invoiceMessage;
    }

    /**
     * @param string $value
     */
    public function setInvoiceMessage($value)
    {
        $this->invoiceMessage = $value;
    }

    /**
     * @return bool
     */
    public function getSendInvoiceLogo()
    {
        return $this->invoiceLogo;
    }

    /**
     * @param bool $value
     */
    public function setSendInvoiceLogo($value)
    {
        $this->invoiceLogo = $value;
    }

    /**
     * @param bool $value
     */
    public function setCreateOrderBeforeRedirect($value)
    {
        $this->createOrderBeforeRedirect = (bool)$value;
    }

    /**
     * @param bool $value
     */
    public function setClearCartBeforeRedirect($value)
    {
        $this->clearCartAfterOrderCreation = (bool)$value;
    }

    /**
     * @return bool
     */
    public function checkConnection()
    {
        return !$this->apiClient->isConnectionFailed();
    }

    /**
     * @param bool $value
     */
    public function setShowLinkInFooter($value)
    {
        $this->showInFooter = $value ? true : false;
    }


    /**
     * @param $value
     */
    public function setB2bSberbankEnabled($value)
    {
        $this->b2bSberbankEnabled = $value;
    }

    /**
     * @param $value
     */
    public function setB2bSberbankPaymentPurpose($value)
    {
        $this->b2bSberbankPaymentPurpose = $value;
    }


    /**
     * @param $value
     */
    public function setB2bSberbankDefaultTaxRate($value)
    {
        $this->b2bSberbankDefaultTaxRate = $value;
    }


    /**
     * @param $value
     */
    public function setB2bTaxRates($value)
    {
        $this->b2bTaxRates = $value;
    }

    /**
     * @return array|null
     */
    public function fetchShopInfo()
    {

        if ($shopInfo = $this->apiClient->fetchShopInfo()) {
            $this->setShopId($shopInfo[ApiClient::SHOP_INFO_ACCOUNT_ID]);
            $this->isTestShop = (bool) $shopInfo[ApiClient::SHOP_INFO_TEST];
            $this->isFiscalizationEnabled = (bool) $shopInfo[ApiClient::SHOP_INFO_FISCALIZATION_ENABLED];
            $this->isSberLoanAvailable =
                isset($shopInfo[ApiClient::SHOP_INFO_PAYMENT_METHODS])
                && in_array(PaymentMethodType::SBER_LOAN, $shopInfo[ApiClient::SHOP_INFO_PAYMENT_METHODS]);
        }
        return $shopInfo;
    }

    /**
     * @return Client
     * @throws Exception
     */
    public function getApiClient()
    {
        return $this->apiClient->getClient();
    }

    /**
     * @return bool
     */
    public function isSberLoanAvailable()
    {
        return $this->isSberLoanAvailable;
    }
}
